#!/usr/bin/env bash
#  Copyright (C) 2022 Seshan Ravikumar
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <https://www.gnu.org/licenses/>.

set -e # Crash when a command fails

echo " * Build Step: OS Variant Image - ProLinux Phone OS "

if  ! { [[ "$PROLINUX_ARCH" == "aarch64" ]] || [[ "$PROLINUX_ARCH" == "x86_64" ]]; }; then
  echo "Invalid PROLINUX_ARCH: $PROLINUX_ARCH"
  exit -1
fi

install_aur_package() {
    echo "Installing AUR package $1 from $2"
    if compgen -G "/artifacts/$PROLINUX_ARCH/pkgs/$1*.pkg.*" > /dev/null; then
        echo "Cached package $1 already exists!"
        cp -v /artifacts/$PROLINUX_ARCH/pkgs/$1*.pkg.* $ROOTFS/home/plu
        chroot $ROOTFS /bin/bash <<EOF
          pacman -U /home/plu/$1*.pkg.* --noconfirm
EOF
        return
    fi
    chroot $ROOTFS /bin/bash <<EOF
    echo "* chroot: Hello from inside the chroot!"
    ls -l
    su plu <<EOSU
      cd ~
      echo $1
      git clone $2
      cd $1
      MAKEFLAGS="-j$(nproc)" makepkg
      ls -l
EOSU
    echo "Running pacman on new package: "
    pacman -U /home/plu/$1/$1*.pkg.* --noconfirm
EOF
  cp -v $ROOTFS/home/plu/$1/$1*.pkg.* /artifacts/$PROLINUX_ARCH/pkgs/
}

export ROOTFS=/build/rootfs
# Source the global build config
source /buildmeta/buildconfig.sh

mkdir -pv /build/rootfs && cd /build

ls -l $ROOTFS

# Uncompress generic rootfs
if  [[ "$PROLINUX_ARCH" == "aarch64" ]]; then
  tar xvf /artifacts/aarch64/sineware.tar.gz -C ${ROOTFS}
else
  tar xvf /artifacts/sineware.tar.gz -C ${ROOTFS}
fi

ls -l ${ROOTFS}
# Set system style
printf "\nstyle=system" >> $ROOTFS/sineware.ini

# Add files
# todo pull these from upstream instead
cp -rv /build-scripts/files/kwinrc $ROOTFS/etc/xdg/kwinrc
cp -rv /build-scripts/files/applications-blacklistrc $ROOTFS/etc/xdg/applications-blacklistrc
cp -rv /build-scripts/files/kscreenlockerrc $ROOTFS/etc/xdg/kscreenlockerrc
cp -rv /build-scripts/files/kdeglobals $ROOTFS/etc/xdg/kdeglobals

echo "  * Entering Arch Chroot (mobile)"
mount -t proc /proc $ROOTFS/proc/
mount -t sysfs /sys $ROOTFS/sys/
mount --rbind /dev $ROOTFS/dev/
cp /etc/resolv.conf $ROOTFS/etc/resolv.conf

chroot $ROOTFS /bin/bash <<EOF
    echo "* chroot: Hello from inside the chroot!"
    pacman -Syy
    pacman -S git base-devel plasma-workspace plasma-wayland-session plasma-nm plasma-pa cmake extra-cmake-modules \
    qt5-tools modemmanager-qt networkmanager-qt powerdevil qmltermwidget qt5-feedback pipewire rust \
    ki18n kirigami2 maliit-keyboard purpose --noconfirm
EOF

mkdir -pv /artifacts/$PROLINUX_ARCH/pkgs/

# Todo eventually we should host a downstream mirror of PKGBUILDs ourselves
install_aur_package plasma-nano https://aur.archlinux.org/plasma-nano.git
install_aur_package qqc2-breeze-style https://aur.archlinux.org/qqc2-breeze-style.git
install_aur_package plasma-settings https://aur.archlinux.org/plasma-settings.git
install_aur_package kirigami-addons https://aur.archlinux.org/kirigami-addons.git
install_aur_package plasma-mobile https://aur.archlinux.org/plasma-mobile.git
install_aur_package qmlkonsole https://aur.archlinux.org/qmlkonsole.git
install_aur_package corrosion-git https://aur.archlinux.org/corrosion-git.git
#install_aur_package plasma-angelfish https://aur.archlinux.org/plasma-angelfish.git


chroot $ROOTFS /bin/bash <<EOF
    echo "* chroot: Hello from inside the chroot!"
    # clean up development dependencies
EOF

umount --recursive $ROOTFS/proc/
umount --recursive $ROOTFS/sys/
umount --recursive $ROOTFS/dev/
rm $ROOTFS/etc/resolv.conf


# todo if only tar is asked for
if  [[ "$PROLINUX_MOBILE_TARGET" == "tar" ]]; then
  cd $ROOTFS
  tar -czvf /artifacts/$PROLINUX_ARCH/sineware-phone-os.tar.gz .
  exit 0
fi


# Squash it
mksquashfs $ROOTFS prolinux-phone.squashfs.img

echo "Allocating raw image..."
IMG_NAME="prolinux-phoneos-${SINEWARE_STRING_BUILD}-pinephone.img"
echo ${IMG_NAME}
fallocate -l 8G ${IMG_NAME}

echo "Creating partition table on image..."
sfdisk ${IMG_NAME} <<EOF
label: dos
label-id: 0x7643e815
unit: sectors

/dev/sda1 : start=        2048, size=     524288, type=83, bootable
/dev/sda2 : start=     526336                   , type=83
EOF

echo "* Starting guestfish and creating LVM volumes on image..."
# todo add files (prolinux.ini) to config lvol
guestfish -a ${IMG_NAME} -a prolinux-phone.squashfs.img  <<EOF
run
pvcreate /dev/sda2
vgcreate ProLinuxVG /dev/sda2
lvcreate lvol_grub ProLinuxVG 16
lvcreate lvol_config ProLinuxVG 128
lvcreate lvol_root_a ProLinuxVG 2048
lvcreate lvol_root_b ProLinuxVG 2048
lvcreate-free lvol_data ProLinuxVG 100
lvm-scan true
copy-device-to-device /dev/sdb /dev/ProLinuxVG/lvol_root_a
mkfs ext4 /dev/ProLinuxVG/lvol_config
mkfs ext4 /dev/ProLinuxVG/lvol_data
mkfs ext4 /dev/ProLinuxVG/lvol_grub
list-filesystems
EOF

du -h ${IMG_NAME}

echo " * Installing p-boot to image..."
echo "   * Copying Files"
cp -v /artifacts/aarch64/sineware-initramfs.cpio.gz /build-scripts/files/p-boot/conf/sineware-initramfs.cpio.gz
pushd .
  cd /build-scripts/files/p-boot/conf/
  gunzip sineware-initramfs.cpio.gz
popd

echo "   * Mounting loopback device"
LOOP_DEV=$(losetup -f)
echo "Loop Device: ${LOOP_DEV}"
losetup -f -P ${IMG_NAME}
losetup -l
echo "-"
ls -1 /dev/loop*

dd if=/build-scripts/files/p-boot/p-boot-serial.bin of=${LOOP_DEV} bs=1024 seek=8 status=progress
/build-scripts/files/p-boot/p-boot-conf /build-scripts/files/p-boot/conf ${LOOP_DEV}p1
du -h ${IMG_NAME}
sync
losetup -d "${LOOP_DEV}"
rm /build-scripts/files/p-boot/conf/sineware-initramfs.cpio

gzip -v ${IMG_NAME}
mv -v ${IMG_NAME}.gz /artifacts/aarch64/${IMG_NAME}.gz

echo " * Done!"
